;----------------------------------------------------------------------------

;Title	: NCO-based reference frequency generator
;Version: 1.00
;Tool   : AVR Assembler 2.0
;Target	: ATtiny25
;Fuses  : E=0xFF, H=0xDF, L=0xFF
;Author	: wubblick@yahoo.com

;----------------------------------------------------------------------------

.include "tn25def.inc"

;----------------------------------------------------------------------------

;Constants:

.equ	F_CLK	= 16000000	;Fclk, Hz
.equ	F_NCO	=   500000	;Fnco, Hz
.equ	F_OUT	=  1348000 	;Fout, mHz

;----------------------------------------------------------------------------

;Derived constants:

.equ	N_TMR	= F_CLK / F_NCO
.equ	FREQ	= F_OUT * N_TMR * 0xFFFFFFFF / (1000 * F_CLK)

;----------------------------------------------------------------------------

;Ports definition:

.equ FOUT   = PB0 ;Fout output

;Port B direction:
.equ DIRB   = (1 << FOUT)
;Port B init/pullups:
.equ PUPB   = ~(1 << FOUT)

;----------------------------------------------------------------------------

;Global register variables:

.def	tsreg	= r2		;SREG store

.def	Phase1	= r16		;phase code
.def	Phase2	= r17
.def	Phase3	= r18
.def	Phase4	= r19

.def	Freq1	= r20		;frequency code (delta phase)
.def	Freq2	= r21
.def	Freq3	= r22
.def	Freq4	= r23

.def	Temp	= r22		;Temp register

;----------------------------------------------------------------------------

.CSEG
.org 0

	rjmp	Init		;reset vector

;----------------------------------------------------------------------------

.org OC0Aaddr

;NCO implementation.
;To minimize the execution time, the handler is placed
;at the vector address. This eliminates the RJMP command.

                                ;4      address store
	in	tsreg,SREG	;1	save status register

	add	Phase1,Freq1	;1      Phase = Phase + Freq
	adc	Phase2,Freq2	;1
	adc	Phase3,Freq3    ;1
	adc	Phase4,Freq4    ;1

	brmi	Off		;1/2
On:	sbi	PORTB,FOUT	;2	FOUT = 1
	rjmp	OnOff		;2
Off:	cbi	PORTB,FOUT	;2	FOUT = 0
	nop			;1

OnOff:	out	SREG,tsreg	;1      restore status register
	reti                    ;4	restore address (19 total)

;----------------------------------------------------------------------------

;Timer0 init:

Init:	ldi	Temp,RAMEND	;locate stack	
	out	SPL,Temp

	ldi	Temp,N_TMR - 1	;timer period
	out	OCR0A,Temp
	ldi	Temp,(1<<WGM01)	;CTC	
	out	TCCR0A,Temp
	ldi	Temp,(1<<CS00)	
	out	TCCR0B,Temp	;CK/1
	ldi	Temp,(1<<OCIE0A)	
	out	TIMSK,Temp	;OC interrupt enable

;Port init:

	ldi	Temp,PUPB	;port B init
	out	PORTB,Temp
	ldi	Temp,DIRB	
	out	DDRB,Temp

;Variables init:

	ldi Freq1,byte1(FREQ)	;load frequency code
	ldi Freq2,byte2(FREQ)
	ldi Freq3,byte3(FREQ)
	ldi Freq4,byte4(FREQ)

;Interrupts enable:

	sei			;inerrupts enable

;Main program:

Main:	rjmp Main		;loop

;----------------------------------------------------------------------------
